#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include "rssi_list.h"

extern Element * rssi_list;

unsigned long long get_time(){

    struct timeval time;
    gettimeofday(&time, NULL);
    return (unsigned long long)(time.tv_sec) * 1000 + (unsigned long long)(time.tv_usec) / 1000;
}

void add_value(Deque * list, int value){

    unsigned long long deadline = get_time() + DEFAULT_KEEP_DELAY;
    Rssi_sample *new = malloc(sizeof(Rssi_sample));
    new->rssi_mW= value;
    new->deadline = deadline;
    if(list->tail == NULL){
        list->head = list->tail = new;
    }else{
        (*list).tail->next = new;
        list->tail = new;
    }
}

void clear_outdated_values(Deque * list){

    Rssi_sample *i = list->head;
    Rssi_sample *prev = i;
    while(i != NULL){
        if(i->deadline < get_time()){
            if(i == list->head){
                list->head = i->next;
            }else if(i == list->tail){
                list->tail = prev;
            }else{
                prev->next = i->next;
            }
            free(i);
        }
        prev = i;
        i = i->next;
    }
}

void clear_values(Deque * list){

    Rssi_sample *i = list->head;
    Rssi_sample *next = i;
    while(i != NULL){
        next = i->next;
        free(i);
        i = next;
    }
    list->head = NULL;
    list->tail = NULL;
}

Element * find_mac(Element * list, u_char * mac_value){

    Element *i = list;
    Element *correspond = NULL;
    int same;
    int k;

    while(i != NULL){
        same = 1;
        for(k = 0; k < 6; ++k){
            if(mac_value[k] != i->mac_addr[k])
                same = 0;
        }
        if(same)
            correspond = i;
        i = i->next;
    }
    return correspond;
}

Element * add_element(Element ** list, u_char * mac_value){

    Element *i = *list;
    Element *new = malloc(sizeof(Element));
    int j;
    new->next = NULL;
    new->measurements.head = NULL;
    new->measurements.tail = NULL;
    for(j = 0;j < 7; ++j){
        new->mac_addr[j] = mac_value[j];
    }
    while(i->next != NULL){
        i = i->next;
    }
    i->next = new;
    return *list;
}

void delete_element(Element ** list, Element * e){

    Element *i = *list;
    Element *prev = (*list);
    while(i != e){
        i = i->next;
    }
    prev->next = i->next;
    free(i);
}
