package android.application.lo53.initialization;
import android.content.Context;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
    /**
     *<b>Connexion class to manage the device / server connexion</b>
     * <p>Attributes :
     *  <ul>
     *      <li>User Agent we choose Mozilla version 5.0</li>
     *      <li>the ip of the server</li>
     *      <li>the string to compose the url</li>
     *      <li>an url connection to establish conection to a given url</li>
     *      <li>an url attribute composed by the string url given  </li>
     *      <li>an Http connection to establish a connection in http mode </li>
     *      <li>resCode an integer to know if the connection went well</li>
     *      <li>response a buffer were we stock the data sent by the server</li>
     *   </ul>
     *</p>
     * @since 2016
    */

public class Connexion {
    private final String USER_AGENT = "Mozilla/5.0";
    private String ip;
    private String  urlStr;
    private URLConnection urlConn;
    private URL url;
    private HttpURLConnection httpConn;
    private int resCode = -1;
    private StringBuffer response;
        String mac;

        /**
         * Constructer initialize the ip to ower  server ip
         * and  initialize the string url instance to have an http url towards the server
         */
    public Connexion(){
        ip="192.168.1.202";
        urlStr = "http://"+ip+":5000/";
        mac="";
    }

        /**
         * create an url in order to send the http connection request
         * @param link
         *             the specific function in the server
         */
    private void create_url(String link){
           urlStr = "http://"+ip+":5000/"+link;
    }

        /**
         * establish connection between the device and server with the created url
         * @return code connection : 200 if the connection went well
         */
    private String establish_cnx(){
        try {
            url = new URL(urlStr);
            urlConn = url.openConnection();

            urlConn.setRequestProperty("User-Agent", USER_AGENT);
            if (!(urlConn instanceof HttpURLConnection)) {
                //throw new IOException("URL isn't an http url");
               return "URL isn't an http url";
            }
            //http connection
            httpConn = (HttpURLConnection) urlConn;
            httpConn.setAllowUserInteraction(false);
            httpConn.setInstanceFollowRedirects(true);
           // httpConn.setReadTimeout(1000 /* milliseconds */);
            //httpConn.setConnectTimeout(1000 /* milliseconds */);
            httpConn.setDoInput(true);
            httpConn.setRequestMethod("GET");
            httpConn.connect();
            //code pour assurer cnx : 200 ok, 404 ou autre nop
            resCode = httpConn.getResponseCode();
            //

            if (resCode == HttpURLConnection.HTTP_OK) {
                //lecture

                BufferedReader in = new BufferedReader(new InputStreamReader(httpConn.getInputStream()));
                String inputLine;
                response = new StringBuffer();

                while ((inputLine = in.readLine()) != null) {
                    response.append(inputLine);
                }
                return "200";
            }

        } catch (MalformedURLException e) {
            e.printStackTrace();
            return  "Mal formed URL";
        } catch (IOException e) {
            e.printStackTrace();
            return  "input unknown";
        }
        return "System crached please try again later";
    }

        /**
         * Send data to a given url towards the server
         * @param data
         *          the information requested such as the mac adress or the location coordinations
         * @return code connection : 200 if the connection went well
         */
    private String send_data(String data){
        //String msg="System crached please try again later";
        try {

            url = new URL(urlStr);
            urlConn = url.openConnection();

            urlConn.setRequestProperty("User-Agent", USER_AGENT);
            if (!(urlConn instanceof HttpURLConnection)) {
                //throw new IOException("URL isn't an http url");
                return "URL isn't an http url";
            }
            //http connection
            httpConn = (HttpURLConnection) urlConn;
            httpConn.setAllowUserInteraction(false);
            httpConn.setInstanceFollowRedirects(true);
            httpConn.setRequestMethod("POST");
            httpConn.connect();
            DataOutputStream wr = new DataOutputStream(httpConn.getOutputStream());
            wr.writeBytes(data);
            wr.flush();
            wr.close();
            //code pour assurer cnx : 200 ok, 404 ou autre nop
            //httpConn.getOutputStream().write(1212);
            resCode = httpConn.getResponseCode();
            //

            if (resCode == HttpURLConnection.HTTP_OK) {
                //lecture

                BufferedReader in = new BufferedReader(new InputStreamReader(httpConn.getInputStream()));
                String inputLine;
                response = new StringBuffer();

                while ((inputLine = in.readLine()) != null) {
                    response.append(inputLine);
                }
                return "200";
            }

        } catch (MalformedURLException e) {
            e.printStackTrace();
            return  "Mal formed URL";
        } catch (IOException e) {
            e.printStackTrace();
            return  "input unknown"+e.getCause();
        }
        return "500";
    }
    public String init(){
        create_url("init");
        return establish_cnx();
    }
    public String send_mac(Context cntx) {
        WifiManager manager = (WifiManager) cntx.getSystemService(Context.WIFI_SERVICE);
        String macAddress = manager.getConnectionInfo().getMacAddress();
        mac = macAddress;
        create_url("send_mac");
        return send_data("mac=" + mac);
    }
    public String send_ap( int ip, String name){
        //voire route  sur server
        create_url("create_ap");
        return send_data("&ip="+ip+"&name="+name);
    }
    public String send_loc(float x,float y){

        create_url("send_loc");

        return send_data("x="+x+"&y="+y+"&mac="+mac);
    }
    /**
    *Establish the reconnexion  to the current wifi SSID
    *@param cntx
            * The application context
    *@param state_wifi
            *Wifi status connected(true) disconnected(false)

     */
    public boolean selectSSID(Context cntx, boolean state_wifi){
        boolean found=false;
        try {


            WifiManager wifiManager = (WifiManager) cntx.getSystemService(cntx.WIFI_SERVICE);
            WifiInfo wifiInfo = wifiManager.getConnectionInfo();
            String ssid = wifiInfo.getSSID();
            wifiManager.setWifiEnabled(state_wifi);

            WifiConfiguration conf = new WifiConfiguration();
            conf.SSID = "\"" + ssid + "\"";
            conf.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);
            conf.priority = 0xBADBAD;
            conf.status = WifiConfiguration.Status.ENABLED;
            conf.allowedKeyManagement.set(WifiConfiguration.KeyMgmt.NONE);

            int id = wifiManager.addNetwork(conf);

            wifiManager.disconnect();
            wifiManager.enableNetwork(id, true);
            while (!wifiManager.reconnect());

       /* List<WifiConfiguration> list = wifiManager.getConfiguredNetworks();
        for( WifiConfiguration i : list ) {
            if(i.SSID != null && i.SSID.equals("\"" + ssid + "\"")) {
                wifiManager.disconnect();
                wifiManager.enableNetwork(i.networkId, true);
                wifiManager.saveConfiguration();
                wifiManager.reconnect();

                found=true;
                break;
            }
        }*/
            return true;
        }catch (Exception e){
            return false;
        }
    }
}
