#!/usr/bin/env python3
# -*- coding:utf-8 -*
#

from flask import Flask, url_for, request, session as websession
from flask import abort

from sqlalchemy import func

from models import Fingerprint, Location, RSSISample, RSSIElement, AccessPoint, setup, dbsession
from utils import compute_location, get_rssi_sample


app = Flask(__name__)

@app.route('/')
def root():
    if 'mac_address' in websession.keys():
        mac = websession['mac_address']
    else:
        mac = "None"
    return """Localization server application:<br/>
<dl>
<dt>Send the device's MAC address (current MAC: %s):</dt>
<dd><form method="POST" action="%s">
    Mac: <input type="text" name="mac" />
    <input type="submit" value="Send" />
    </form>
</dd>
<dt>Get loc:</dt>
<dd><a href="%s">LOCATE ME!</a></dd>
<dt>AP list:</dt>
<dd>%s</dd>
</dl>
""" % (
        mac,
        url_for('send_mac'),
        url_for('get_loc'),
        ("<br/>".join([str(ap) for ap in dbsession.query(AccessPoint).all()])),
        )

#côté device :"cnx"
@app.route('/init_loc')
def init():
    """
    Ask the device to send its MAC address for location
    """
    count = dbsession.query(Location.id).count()
    if count < 1:
        return "Database not ready yet", 500
    return "200"

@app.route('/send_mac_loc', methods=['POST'])
def send_mac():
    """
    Get the MAC address of the device that wants to be located
    It would be wonderful if we could use many devices at the same time!

    Send a POST request with the following parameter:
        mac=<your mac address>
    """
    mac = request.form.get('mac')
    print(mac)
    websession["mac_address"] = mac
    return "ACK, got your MAC, now <a href=\"%s\">get your location</a>" % url_for('get_loc')

@app.route('/get_loc')
def get_loc():
    """
    Get the location of the device
    """
    print("Locating...")
    if "mac_address" in websession.keys():
        mac = websession["mac_address"]
    else:
        mac = request.args.get('mac')
    rssi_sample = get_rssi_sample(mac)
    location = compute_location(rssi_sample)

    # Process: ask the AP for the RSSI Samples for the saved MAC address (in websession) and compute the found RSSI Sample,
    # then compare it to the ones in the DB to get the nearest location
    return "Located! You are here: %0.2f, %0.2f" % (location.x,location.y)

# No need for additionnal routes: as soon as we have some values in the DB, we can start locating clients with the
# other server app.
# The more we have values (fingerprints), the more accurate the location will be!

if __name__ == '__main__':
    app.secret_key = ';kj;dmqclakmxa;krwjcm;almx;lmjr,x;a,lkajerox3u4m2qp34ix2i3jke,2pioxi43z'
    app.run(debug=True, host='192.168.1.202', port=5001)

