#!/usr/bin/env python3
# -*- coding:utf-8 -*
#

from flask import Flask, request, session as websession, url_for
from flask import abort

from models import Fingerprint, Location, RSSISample, RSSIElement, AccessPoint, setup, dbsession
from utils import get_rssi_sample

loc_x=0
loc_y=0
app = Flask(__name__)

@app.route('/')
def root():
    if 'mac_address' in websession.keys():
        mac = websession['mac_address']
    else:
        mac = "None"
    return """Set up server application:<br/>
<dl>
<dt>Initialze the DB:</dt>
<dd><a href="%s">Init</a></dd>
<dt>Send the device's MAC address (current MAC: %s):</dt>
<dd><form method="POST" action="%s">
    Mac: <input type="text" name="mac" />
    <input type="submit" value="Send" />
    </form>
</dd>
<dt>Add an AP:</dt>
<dd><form method="POST" action="%s">
    Name: <input type="text" name="name" />
    IP: <input type="text" name="ip_address" />
    <input type="submit" value="Send" />
    </form>
</dd>
<dt>Send localisation:</dt>
<dd><form method="POST" action="%s">
    X: <input type="text" name="x" />
    Y: <input type="text" name="y" />
    <input type="submit" value="Send" />
    </form>
</dd>
<dt>AP list:</dt>
<dd>%s</dd>
</dl>
""" % (
        url_for('init'),
        mac,
        url_for('send_mac'),
        url_for('create_ap'),
        url_for('send_loc'),
        ("<br/>".join([str(ap) for ap in dbsession.query(AccessPoint).all()])),
        )


@app.route('/init')
def init():
    """
    Set up the database before the mapping of the building
    """
    setup()
    return """Ready for setup, send me your MAC or <a href="%s">go back</a><br/><form method="POST" action="%s"><input
type="text" name="mac" /><input type="submit" value="Send" />""" % (url_for('root'), url_for('send_mac'))

@app.route('/send_mac', methods=['POST'])
def send_mac():
    """
    Get the MAC address of the device that does the mapping
    It would be wonderful if we could use many devices at the same time!

    Send a POST request with the following parameter:
        mac=<your mac address>
    """
    mac_address = request.form.get('mac')
    websession["mac_address"] = mac_address
    return """ACK, you can send me some locations on the <a href="%s">main page</a>""" % (url_for('root'))

@app.route('/create_ap', methods=['POST'])
def create_ap():
    dbsession.rollback()
    name = request.form.get('name')
    ip_address = str(request.form.get('ip_address'))
    ap = AccessPoint(name = name, ip_address = ip_address)
    dbsession.add(ap)
    dbsession.commit()
    return """ACK, AP saved. <a href="%s">Main page</a>""" % (url_for('root'))

@app.route('/send_loc', methods=['POST'])
def send_loc():
    """
    Get the location of the device

    Send a POST request with the following parameters:
        x=<your x coordinate>
        y=<your y coordinate>
    """
    try:
        loc_x = float(request.form.get('x'))
        loc_y = float(request.form.get('y'))
        loc = Location(x=loc_x, y=loc_y)
        dbsession.add(loc)
        dbsession.commit()
    except:
        return "Bad request, you need to send x and y as float", 500
    f = Fingerprint(location_id=loc.id) # Create a new fingerprint
    dbsession.add(f)
    dbsession.commit() # Commit to get an id
    r = RSSISample(fingerprint_id=f.id) # Create a new RSSI sample
    dbsession.add(r)
    dbsession.commit() # Commit to get an id

    if "mac_address" in websession.keys():
        mac = websession["mac_address"]
    else:
        mac = request.args.get('mac')
    rssi = get_rssi_sample(mac) # Get an rssi sample from all the APs
    for (ip, value) in rssi.items():
        dbsession.add(RSSIElement(ip=ip, value=value, rssi_sample_id=r.id)) # Store it
    dbsession.commit()
    return """Located and saved, thank you! You can now move and send a new location
on the <a href="%s">Main page</a> <br/>RSSI: %s""" % (url_for('root'), str(rssi))

# No need for additionnal routes: as soon as we have some values in the DB, we can start locating clients with the
# other server app.
# The more we have values (fingerprints), the more accurate the location will be!

if __name__ == '__main__':
    app.secret_key = 'pnmrcmfx;lwime,2j49coisjdxlp9kyuh1j<F3>xm'
    app.run(debug=True, host='192.168.1.202')

