#!/usr/bin/env python3
# -*- coding:utf-8 -*

from threading import Thread
from math import sqrt

import random
import time
import socket
import binascii
import struct

from models import Fingerprint, RSSISample, RSSIElement, AccessPoint, Location, dbsession

PORT=8888


def compute_location(rssi_sample):
    """
    Get a location given an rssi sample (as RSSISample class or dict with MAC as keys and RSSI values as values)
    """
    #var needed for the computation :
    min_dist = 1111111111111
    threshold = 60
    locat = Location(x=0., y=0.)
    fingerprints = dbsession.query(Fingerprint).all()
    for f in fingerprints:
        dist = rssi_distance(rssi_sample, f.rssi_samples, threshold)
        if min_dist > dist:
            min_dist = dist
            locat = f.location
    return locat

def rssi_distance(s1, s2, threshold):
    """
    With the magic of RSSISample's dict functions, we just handle s1 and s2 as dicts of RSSIElement
    """
    distance = 0
    for (ip, value) in s1.items():
        if ip in s2.keys() :
            distance += (value - s2[ip]) ** 2
        else:
            distance += (value + threshold) ** 2
    for (ip, value) in s2.items():
        if ip not in s1.keys() :
            distance += (value + threshold) ** 2
    return sqrt(distance)

def get_rssi_sample(device_mac):
    """
    Get the RSSI sample for the given device mac address
    Returns an RSSISample object
    """
    thread_list = []
    elem_dict = {}
    ap_list = dbsession.query(AccessPoint).all()
    for ap in ap_list:
        thread_list.append(Thread(target=get_rssi_element, args=(ap.ip_address, device_mac, elem_dict)))
    for t in thread_list:
        t.start()
    for t in thread_list:
        t.join()
    return elem_dict

def get_rssi_element(ip, mac, elem_dict):
    """
    Get the RSSI element for the given AP ip address and the given device mac address
    Returns an RSSIElement object
    """
    mac = mac.replace(':', '')
    macbytes = binascii.unhexlify(mac)
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.settimeout(3)
    try:
        s.connect((str(ip), PORT))
        sent = 0
        while sent == 0:
            sent = s.send(macbytes)

        chunk = b''
        while chunk == b'':
            chunk = s.recv(4)

        s.shutdown(socket.SHUT_RDWR)
        s.close()
        print("%s: %s" % (str(ip), struct.unpack('!i', chunk)))

        rssi = struct.unpack('!i', chunk)[0]
    except:
        rssi = -100

    elem_dict[ip] = rssi



